@@ -258,6 +258,13 @@ is_valid_nonce_tcp (const gchar  *address_entry,
       goto out;
     }
 
+  if (host != NULL)
+    {
+      /* TODO: validate host */
+    }
+
+  nonce_file = nonce_file; /* To avoid -Wunused-but-set-variable */
+
   ret= TRUE;
 
  out:
@@ -332,6 +339,11 @@ is_valid_tcp (const gchar  *address_entry,
       goto out;
     }
 
+  if (host != NULL)
+    {
+      /* TODO: validate host */
+    }
+
   ret= TRUE;
 
  out:
@@ -568,7 +580,7 @@ g_dbus_address_connect (const gchar   *address_entry,
     {
       const gchar *s;
       const gchar *host;
-      guint port;
+      glong port;
       gchar *endp;
       gboolean is_nonce;
 
@@ -932,7 +944,7 @@ g_dbus_address_get_stream_sync (const gchar   *address,
   last_error = NULL;
 
   addr_array = g_strsplit (address, ";", 0);
-  if (addr_array[0] == NULL)
+  if (addr_array != NULL && addr_array[0] == NULL)
     {
       last_error = g_error_new_literal (G_IO_ERROR,
                                         G_IO_ERROR_INVALID_ARGUMENT,
@@ -1277,7 +1289,7 @@ g_dbus_address_get_for_bus_sync (GBusType       bus_type,
         {
           g_print ("GDBus-debug:Address: Cannot look-up address bus type `%s': %s\n",
                    _g_dbus_enum_to_string (G_TYPE_BUS_TYPE, bus_type),
-                   local_error->message);
+                   local_error ? local_error->message : "");
         }
       _g_dbus_debug_print_unlock ();
     }
