@@ -31,7 +31,6 @@
 #include "gdelayedsettingsbackend.h"
 #include "gsettingsbackendinternal.h"
 #include "gsettings-mapping.h"
-#include "gio-marshal.h"
 #include "gsettingsschema.h"
 
 #include <string.h>
@@ -141,16 +140,16 @@
  * <programlisting><![CDATA[
  * <schemalist>
  *
- *   <enum id="myenum">
+ *   <enum id="org.gtk.Test.myenum">
  *     <value nick="first" value="1"/>
  *     <value nick="second" value="2"/>
  *   </enum>
  *
- *   <enum id="myflags">
+ *   <flags id="org.gtk.Test.myflags">
  *     <value nick="flag1" value="1"/>
  *     <value nick="flag2" value="2"/>
  *     <value nick="flag3" value="4"/>
- *   </enum>
+ *   </flags>
  *
  *   <schema id="org.gtk.Test">
  *
@@ -172,11 +171,11 @@
  *       <default>'Joe'</default>
  *     </key>
  *
- *     <key name='enumerated-key' enum='myenum'>
+ *     <key name='enumerated-key' enum='org.gtk.Test.myenum'>
  *       <default>'first'</default>
  *     </key>
  *
- *     <key name='flags-key' flags='myflags'>
+ *     <key name='flags-key' flags='org.gtk.Test.myflags'>
  *       <default>["flag1",flag2"]</default>
  *     </key>
  *   </schema>
@@ -622,7 +621,7 @@ g_settings_class_init (GSettingsClass *class)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GSettingsClass, change_event),
                   g_signal_accumulator_true_handled, NULL,
-                  _gio_marshal_BOOL__POINTER_INT,
+                  g_cclosure_marshal_generic,
                   G_TYPE_BOOLEAN, 2, G_TYPE_POINTER, G_TYPE_INT);
 
   /**
@@ -676,7 +675,7 @@ g_settings_class_init (GSettingsClass *class)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GSettingsClass, writable_change_event),
                   g_signal_accumulator_true_handled, NULL,
-                  _gio_marshal_BOOLEAN__UINT, G_TYPE_BOOLEAN, 1, G_TYPE_UINT);
+                  g_cclosure_marshal_generic, G_TYPE_BOOLEAN, 1, G_TYPE_UINT);
 
   /**
    * GSettings:context:
