@@ -54,7 +54,6 @@
 #include "gioerror.h"
 #include "gioenums.h"
 #include "gioerror.h"
-#include "gio-marshal.h"
 #include "gnetworkingprivate.h"
 #include "gsocketaddress.h"
 #include "gsocketcontrolmessage.h"
@@ -2109,7 +2108,7 @@ g_socket_shutdown (GSocket   *socket,
 
   g_return_val_if_fail (G_IS_SOCKET (socket), TRUE);
 
-  if (!check_socket (socket, NULL))
+  if (!check_socket (socket, error))
     return FALSE;
 
   /* Do nothing? */
@@ -2136,7 +2135,7 @@ g_socket_shutdown (GSocket   *socket,
     {
       int errsv = get_socket_errno ();
       g_set_error (error, G_IO_ERROR, socket_io_error_from_errno (errsv),
-		   _("Unable to create socket: %s"), socket_strerror (errsv));
+		   _("Unable to shutdown socket: %s"), socket_strerror (errsv));
       return FALSE;
     }
 
@@ -2196,7 +2195,7 @@ g_socket_close (GSocket  *socket,
   if (socket->priv->closed)
     return TRUE; /* Multiple close not an error */
 
-  if (!check_socket (socket, NULL))
+  if (!check_socket (socket, error))
     return FALSE;
 
   while (1)
@@ -2537,7 +2536,7 @@ static GSourceFuncs socket_source_funcs =
   socket_source_dispatch,
   socket_source_finalize,
   (GSourceFunc)socket_source_closure_callback,
-  (GSourceDummyMarshal)_gio_marshal_BOOLEAN__FLAGS,
+  (GSourceDummyMarshal)g_cclosure_marshal_generic,
 };
 
 static GSource *
@@ -3290,6 +3289,14 @@ g_socket_receive_message (GSocket                 *socket,
     else
       msg.msg_flags = 0;
 
+    /* We always set the close-on-exec flag so we don't leak file
+     * descriptors into child processes.  Note that gunixfdmessage.c
+     * will later call fcntl (fd, FD_CLOEXEC), but that isn't atomic.
+     */
+#ifdef MSG_CMSG_CLOEXEC
+    msg.msg_flags |= MSG_CMSG_CLOEXEC;
+#endif
+
     /* do it */
     while (1)
       {
@@ -3299,6 +3306,14 @@ g_socket_receive_message (GSocket                 *socket,
 	  return -1;
 
 	result = recvmsg (socket->priv->fd, &msg, msg.msg_flags);
+#ifdef MSG_CMSG_CLOEXEC	
+	if (result < 0 && get_socket_errno () == EINVAL)
+	  {
+	    /* We must be running on an old kernel.  Call without the flag. */
+	    msg.msg_flags &= ~(MSG_CMSG_CLOEXEC);
+	    result = recvmsg (socket->priv->fd, &msg, msg.msg_flags);
+	  }
+#endif
 
 	if (result < 0)
 	  {
@@ -3515,11 +3530,16 @@ g_socket_get_credentials (GSocket   *socket,
 
   ret = NULL;
 
-#ifdef __linux__
+#if defined(__linux__) || defined(__OpenBSD__)
   {
-    struct ucred native_creds;
     socklen_t optlen;
+#if defined(__linux__)
+    struct ucred native_creds;
     optlen = sizeof (struct ucred);
+#elif defined(__OpenBSD__)
+    struct sockpeercred native_creds;
+    optlen = sizeof (struct sockpeercred);
+#endif
     if (getsockopt (socket->priv->fd,
                     SOL_SOCKET,
                     SO_PEERCRED,
@@ -3537,7 +3557,11 @@ g_socket_get_credentials (GSocket   *socket,
       {
         ret = g_credentials_new ();
         g_credentials_set_native (ret,
+#if defined(__linux__)
                                   G_CREDENTIALS_TYPE_LINUX_UCRED,
+#elif defined(__OpenBSD__)
+                                  G_CREDENTIALS_TYPE_OPENBSD_SOCKPEERCRED,
+#endif
                                   &native_creds);
       }
   }
