@@ -24,7 +24,6 @@
 #include "gtlsconnection.h"
 #include "gcancellable.h"
 #include "gioenumtypes.h"
-#include "gio-marshal.h"
 #include "gsocket.h"
 #include "gtlsbackend.h"
 #include "gtlscertificate.h"
@@ -195,8 +194,8 @@ g_tls_connection_class_init (GTlsConnectionClass *klass)
    * GTlsConnection:peer-certificate-errors:
    *
    * The errors noticed-and-ignored while verifying
-   * #GTlsConnection:peer-certificate. Normally this should be %0, but
-   * it may not be if #GTlsClientConnection::validation-flags is not
+   * #GTlsConnection:peer-certificate. Normally this should be 0, but
+   * it may not be if #GTlsClientConnection:validation-flags is not
    * %G_TLS_CERTIFICATE_VALIDATE_ALL, or if
    * #GTlsConnection::accept-certificate overrode the default
    * behavior.
@@ -265,7 +264,7 @@ g_tls_connection_class_init (GTlsConnectionClass *klass)
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GTlsConnectionClass, accept_certificate),
 		  g_signal_accumulator_true_handled, NULL,
-		  _gio_marshal_BOOLEAN__OBJECT_FLAGS,
+		  g_cclosure_marshal_generic,
 		  G_TYPE_BOOLEAN, 2,
 		  G_TYPE_TLS_CERTIFICATE,
 		  G_TYPE_TLS_CERTIFICATE_FLAGS);
@@ -569,7 +568,7 @@ g_tls_connection_set_rehandshake_mode (GTlsConnection       *conn,
  * @conn: a #GTlsConnection
  *
  * Gets @conn rehandshaking mode. See
- * g_tls_connection_set_rehandshake() for details.
+ * g_tls_connection_set_rehandshake_mode() for details.
  *
  * Return value: @conn's rehandshaking mode
  *
